#include "mainwindow.h"
#include "ui_mainwindow.h"

QTM_USE_NAMESPACE

/**Initializes the application and its widgets.*/
MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    //initialize

    //set plugin path
    QDir dir;
    dir.cd(QCoreApplication::applicationDirPath());
    dir.cd("plugins");
    QCoreApplication::addLibraryPath(dir.absolutePath());

    //create program folder in home directory
    if(!QDir::home().cd("itlogger")){
        QDir::home().mkdir("itlogger");
    }

    //ui Setup
    ui->setupUi(this);
    ui->statusBar->showMessage(tr("Initializing..."),0);

    //hide options
    ui->scrollArea->setVisible(false);

    //thumbnailviewer init
    thumbviewer = new thumbnailviewer(this,ui->graphicsView);
    imageViewer = new QDockWidget();

    QWidget* lTitleBar = imageViewer->titleBarWidget();
    QWidget* lEmptyWidget = new QWidget();
    imageViewer->setTitleBarWidget(lEmptyWidget);
    delete lTitleBar;

    pictureLabel = new QLabel();

    //explorer init
    exp = new explorer(ui->listWidget,thumbviewer);

    //plotter init
    plotter[0] = new plot(ui->coordPlot,3,true,tr("Altitude [m]"),
                          "","",tr("Altitude"), false);
    plotter[1] = new plot(ui->speedPlot,3,true,tr("Speed [m/s]"),
                          tr("Speed"),tr("Course"),"", true);
    plotter[2] = new plot(ui->accPlot,3,true,tr("Acceleration [g]"),
                          tr("X-Direction"),tr("Y-Direction"),tr("Z-Direction"), false);
    plotter[3] = new plot(ui->angAccPlot,3,true,tr("Angular Acceleration [rad/s]"),
                          tr("X-Axis"),tr("Y-Axis"),tr("Z-Axis"),false);
    plotter[4] = new plot(ui->OBDPlot,8,false,tr("Speed [km/h]"),
                          tr("Speed"),tr("Throttle position"),tr("Engine RPM"),false);

    //disable unused curves
    plotter[0]->setVisible(false,0);
    plotter[0]->setVisible(false,1);
    plotter[1]->setVisible(false,2);

    //add course axis to plotter1
    plotter[1]->setAxis(tr("Course [degree]"),1);

    //set obd curve names
    plotter[4]->setCurveTitle(tr("Actual engine - percent torque"),3);
    plotter[4]->setCurveTitle(tr("Engine reference torque"),4);
    plotter[4]->setCurveTitle(tr("Intake air temperature"),5);
    plotter[4]->setCurveTitle(tr("Ambient air temperature"),6);
    plotter[4]->setCurveTitle(tr("Barometric pressure"),7);

    //printer init
    printer = new QPrinter(QPrinter::HighResolution);

    //create language menu
    this->createLanguageMenu();

    //map init
    geoprovider = new QGeoServiceProvider("openstreetmap");
    if (geoprovider->error() != QGeoServiceProvider::NoError)
        qDebug("plugin not found");

    manager = geoprovider->mappingManager();
    gpsmap = new map(manager);
    gpsmap->init(ui->map_view);

    //xdata
    xdata = NULL;

    //not loading right now
    loading = false;

    //init widget splitter
    splitter = new QSplitter;
    splitter->setChildrenCollapsible(false);
    splitter->setOrientation(Qt::Horizontal);
    splitter->addWidget(ui->listWidget);
    splitter->addWidget(ui->widget);
    splitter->setStretchFactor(0,0);
    splitter->setStretchFactor(1,1);
    setCentralWidget(splitter);
    splitter->show();

    this->installEventFilter(this);

    data_loaded = false;

    //importhandler init
    ui->statusBar->showMessage(tr("Importing data..."),0);
    import = new importhandler();

    //init done
    exp->refresh();
    ui->statusBar->showMessage(tr("Ready"),0);
}

/**Destroys the MainWindow.*/
MainWindow::~MainWindow()
{
    delete ui;
}

/**Quits the application.*/
void MainWindow::on_actionBeenden_triggered()
{
    this->saveConfig();
    qApp->quit();
}

/**Saves configuration when the application closes.*/
void MainWindow::closeEvent(QCloseEvent *)
{
    this->saveConfig();
}

/**Shows the about dialog.*/
void MainWindow::on_actionAbout_triggered()
{
    about *about_dialog = new about();
    about_dialog->exec();
}

/**Loads data and displays it.*/
void MainWindow::on_actionLaden_triggered()
{
    load();
}

/**Calls the clone function of the explorer.*/
void MainWindow::on_actionCopy_triggered()
{
    ui->statusBar->showMessage(tr("Copying data..."),0);
    exp->clone();
    ui->statusBar->showMessage(tr("Ready"),0);
}

/**Calls the merge function of the explorer.*/
void MainWindow::on_actionMerge_triggered()
{
    ui->statusBar->showMessage(tr("Merging data..."),0);
    exp->merge();
    ui->statusBar->showMessage(tr("Ready"),0);
}

/**Loads and displays data on doubleclick.*/
void MainWindow::on_listWidget_doubleClicked()
{
    load();
}

/**Opens the data import dialog.*/
void MainWindow::on_actionImportieren_triggered()
{
    import->importFromFile();
    exp->refresh();
}

/**Prints the application window.*/
void MainWindow::on_actionDrucken_triggered()
{
    //printer setup
    QPrintDialog *dialog = new QPrintDialog(printer);
    if(dialog->exec())
    {
        //pressed 'print'
        QPainter painter;
        painter.begin(printer);

        //scale size to fullscreen
        double xscale = printer->pageRect().width()/double(ui->centralWidget->width());
        double yscale = printer->pageRect().height()/double(ui->centralWidget->height());
        double scale = qMin(xscale, yscale);
        painter.translate(printer->paperRect().x() + printer->pageRect().width()/2,
        printer->paperRect().y() + printer->pageRect().height()/2);
        painter.scale(scale, scale);
        painter.translate(-width()/2, -height()/2);

        //print
        ui->centralWidget->render(&painter);
    }

    delete dialog;
}

/**Delete a single element from explorer.*/
void MainWindow::on_actionL_schen_triggered()
{
    exp->deleteEntry();
}

/**Show export data dialog.*/
void MainWindow::on_actionExport_triggered()
{
    //create the dialog
    expdialog = new exportdialog;

    //no data is loaded
    if(!data_loaded)
    {
        QMessageBox *msg = new QMessageBox;
        msg->setText(tr("No data is loaded"));
        msg->setIcon(QMessageBox::Warning);
        msg->setWindowTitle(QObject::tr("Error"));
        msg->setWindowIcon(QIcon(":/images/icon.png"));
        msg->exec();
        delete msg;
    }

    //data is loaded
    else
    {
        expdialog->setData(exp->data, ui->spinCoordBegin->value(), ui->spinCoordEnd->value());
        expdialog->exec();
    }

    delete expdialog;
}

/**Show help dialog.*/
void MainWindow::on_actionHilfe_triggered()
{
    help *helper = new help;
    QFile file(QDir::home().absolutePath()+"/itlogger/config.cnf");
    if(file.open(QIODevice::ReadOnly))
    {
        //load language
        QString lang;
        QTextStream in(&file);
        in>>lang;
        file.close();
        lang = lang.left(lang.indexOf(","));

        if(helper->setLanguage(lang.toLower()))
            helper->show();
        else{
            QMessageBox *msg = new QMessageBox;
            msg->setText("There is no help file for the selected language.");
            msg->setIcon(QMessageBox::Warning);
            msg->setWindowTitle("Error");
            msg->exec();
            delete msg;
            delete helper;
        }
    }
    else{
        this->createConfig();
        this->on_actionHilfe_triggered();
    }
}

/**Show or hide options.*/
void MainWindow::on_actionOptions_triggered()
{
    if(ui->scrollArea->isVisible())
    {
        ui->scrollArea->setVisible(false);
        ui->pushButton->setIcon(QIcon(":/images/arrow_left.png"));
        ui->pushButton->setChecked(false);
    }

    else
    {
        ui->scrollArea->setVisible(true);
        ui->pushButton->setIcon(QIcon(":/images/arrow_right.png"));
        ui->pushButton->setChecked(true);
    }
}

/**Show or hide options.*/
void MainWindow::on_pushButton_clicked(bool checked)
{
    if(checked)
    {
        ui->scrollArea->setVisible(true);
        ui->pushButton->setIcon(QIcon(":/images/arrow_right.png"));
    }

    else
    {
        ui->scrollArea->setVisible(false);
        ui->pushButton->setIcon(QIcon(":/images/arrow_left.png"));
    }
    thumbviewer->updateView(ui->graphicsView->horizontalScrollBar()->sliderPosition()+310);
}

/**Creates the language menu using the .qm files in the languages folder.*/
void MainWindow::createLanguageMenu(void)
{
    QActionGroup* langGroup = new QActionGroup(ui->menuLanguage);;

    connect(langGroup,SIGNAL(triggered(QAction *)),this,SLOT(slotLanguageChange(QAction *)));

    //searches the languages path for .qm files
    QString langPath = QCoreApplication::applicationDirPath();
    langPath.append("/languages");

    QDir dir(langPath);
    QStringList fileNames = dir.entryList(QStringList("*.qm"));

    //creates and action for each file
    for(int i=0;i<fileNames.size();i++)
    {
        QString lang;
        lang = fileNames[i];
        lang.truncate(lang.lastIndexOf("."));
        QAction *action = new QAction(lang,this);
        action->setData(lang);

        ui->menuLanguage->addAction(action);
        langGroup->addAction(action);
    }

    this->loadConfig();
}

/**Loads the language, window size, line colors and line thicknesses from the config file.*/
void MainWindow::loadConfig()
{
    QFile file(QDir::home().absolutePath()+"/itlogger/config.cnf");
    if(file.open(QIODevice::ReadOnly))
    {
        //load language
        QString lines;
        QTextStream in(&file);
        in>>lines;
        file.close();
        QString lang = lines.left(lines.indexOf(","));

        lang.prepend("languages/");
        lang.prepend(QCoreApplication::applicationDirPath() + "/");
        lang.append(".qm");
        translator.load(lang);
        qApp->installTranslator(&translator);
        ui->retranslateUi(this);

        //load window size
        lines.remove(0,lines.indexOf(",")+1);
        if(lines.left(1)=="-"){
            this->showMaximized();
            lines.remove(0,2);
        }
        else{
            int width = lines.left(lines.indexOf("x")).toInt();
            lines.remove(0,lines.indexOf("x")+1);
            int heigth = lines.left(lines.indexOf(",")).toInt();
            this->resize(width,heigth);
            lines.remove(0,lines.indexOf(",")+1);
        }

        //load colors and thicknesses
        bool ok;
        ui->comboAltitudeColor->setCurrentIndex(lines.left(1).toInt(&ok));
        if(ok==false){
            this->restoreConfig();
            return;
        }
        lines.remove(0,1);
        ui->comboAltitudeThickness->setCurrentIndex(lines.left(1).toInt());
        lines.remove(0,1);
        ui->comboXSpeedColor->setCurrentIndex(lines.left(1).toInt());
        lines.remove(0,1);
        ui->comboYSpeedColor->setCurrentIndex(lines.left(1).toInt());
        lines.remove(0,1);
        ui->comboXSpeedThickness->setCurrentIndex(lines.left(1).toInt());
        lines.remove(0,1);
        ui->comboYSpeedThickness->setCurrentIndex(lines.left(1).toInt());
        lines.remove(0,1);
        ui->comboXAccColor->setCurrentIndex(lines.left(1).toInt());
        lines.remove(0,1);
        ui->comboYAccColor->setCurrentIndex(lines.left(1).toInt());
        lines.remove(0,1);
        ui->comboZAccColor->setCurrentIndex(lines.left(1).toInt());
        lines.remove(0,1);
        ui->comboXAccThickness->setCurrentIndex(lines.left(1).toInt());
        lines.remove(0,1);
        ui->comboYAccThickness->setCurrentIndex(lines.left(1).toInt());
        lines.remove(0,1);
        ui->comboZAccThickness->setCurrentIndex(lines.left(1).toInt());
        lines.remove(0,1);
        ui->comboXAngAccColor->setCurrentIndex(lines.left(1).toInt());
        lines.remove(0,1);
        ui->comboYAngAccColor->setCurrentIndex(lines.left(1).toInt());
        lines.remove(0,1);
        ui->comboZAngAccColor->setCurrentIndex(lines.left(1).toInt());
        lines.remove(0,1);
        ui->comboXAngAccThickness->setCurrentIndex(lines.left(1).toInt());
        lines.remove(0,1);
        ui->comboYAngAccThickness->setCurrentIndex(lines.left(1).toInt());
        lines.remove(0,1);
        ui->comboZAngAccThickness->setCurrentIndex(lines.left(1).toInt());
        lines.remove(0,1);
        ui->comboOBDColor->setCurrentIndex(lines.left(1).toInt());
        lines.remove(0,1);
        ui->comboOBDThickness->setCurrentIndex(lines.left(1).toInt());
        lines.remove(0,1);
        ui->comboOBDValue->setCurrentIndex(lines.toInt(&ok));
        if(ok==false)
            this->restoreConfig();
    }
    //create a new config file when it couldn't be opened
    else
        this->createConfig();
}

/**Restores the config file when a read error occured.*/
void MainWindow::restoreConfig(){
    QFile file(QDir::home().absolutePath()+"/itlogger/config.cnf");
    file.remove();
    file.close();
    this->createConfig();
}

/**Saves the current language and line settings to the config file, called when the application is closed.*/
void MainWindow::saveConfig(){
    QFile file(QDir::home().absolutePath()+"/itlogger/config.cnf");
    if(file.open(QIODevice::ReadOnly))
    {
        QString values;
        QTextStream in(&file);
        in>>values;
        file.close();

        values.truncate(values.indexOf(",")+1);
        if(this->isMaximized())
            values.append("-,");
        else
            values.append(QString::number(this->width())+"x"+QString::number(this->height())+",");

        values.append(QString::number(ui->comboAltitudeColor->currentIndex()));
        values.append(QString::number(ui->comboAltitudeThickness->currentIndex()));
        values.append(QString::number(ui->comboXSpeedColor->currentIndex()));
        values.append(QString::number(ui->comboYSpeedColor->currentIndex()));
        values.append(QString::number(ui->comboXSpeedThickness->currentIndex()));
        values.append(QString::number(ui->comboYSpeedThickness->currentIndex()));
        values.append(QString::number(ui->comboXAccColor->currentIndex()));
        values.append(QString::number(ui->comboYAccColor->currentIndex()));
        values.append(QString::number(ui->comboZAccColor->currentIndex()));
        values.append(QString::number(ui->comboXAccThickness->currentIndex()));
        values.append(QString::number(ui->comboYAccThickness->currentIndex()));
        values.append(QString::number(ui->comboZAccThickness->currentIndex()));
        values.append(QString::number(ui->comboXAngAccColor->currentIndex()));
        values.append(QString::number(ui->comboYAngAccColor->currentIndex()));
        values.append(QString::number(ui->comboZAngAccColor->currentIndex()));
        values.append(QString::number(ui->comboXAngAccThickness->currentIndex()));
        values.append(QString::number(ui->comboYAngAccThickness->currentIndex()));
        values.append(QString::number(ui->comboZAngAccThickness->currentIndex()));
        values.append(QString::number(ui->comboOBDColor->currentIndex()));
        values.append(QString::number(ui->comboOBDThickness->currentIndex()));
        values.append(QString::number(ui->comboOBDValue->currentIndex()));

        file.open(QIODevice::Truncate | QIODevice::WriteOnly);
        QTextStream out(&file);
        out<<values;
        file.close();
    }
    else
        this->createConfig();
}

/**Creates a new config file when none was found.*/
void MainWindow::createConfig()
{
    QMessageBox *msg = new QMessageBox;
    msg->setWindowTitle(tr("Error"));
    msg->setText(tr("Couldn't find configuration file. A new one was created using default values."));
    msg->setWindowIcon(QIcon(":/images/icon.png"));
    msg->setIcon(QMessageBox::Warning);
    msg->exec();
    delete msg;

    QFile *file = new QFile(QDir::home().absolutePath()+"/itlogger/config.cnf");
    file->open(QIODevice::WriteOnly | QIODevice::Text);
    QString defaultValues = "English,800x600,010111012111012111010";
    QTextStream out(file);
    out<<defaultValues;
    file->close();
    delete file;

    this->loadConfig();
}

/**Is called when the language is changed.*/
void MainWindow::slotLanguageChange(QAction *action)
{
    //saves diagram line settings
    this->saveConfig();

    //saves the new language to the config file
    QFile file(QDir::home().absolutePath()+"/itlogger/config.cnf");
    file.open(QIODevice::ReadOnly);
    QString values;
    QTextStream in(&file);
    in>>values;
    file.close();

    values.remove(0,values.indexOf(","));
    values.prepend(action->data().toString());

    file.open(QIODevice::Truncate | QIODevice::WriteOnly);
    QTextStream out(&file);
    out<<values;
    file.close();

    //reloads the ui
    this->loadConfig();
}

/**Show printer setup dialog.*/
void MainWindow::on_actionDruckereinstellungen_triggered()
{
    QPageSetupDialog *dialog = new QPageSetupDialog(printer,0);
    dialog->exec();
    delete dialog;
}

/**Loads data, hands it over to the plotters.*/
int MainWindow::load()
{
    //check if loading
    if(!loading)
    {
        loading = true;
        data_loaded = true;
        ui->statusBar->showMessage(tr("Loading data..."),0);

        //set the data for the respective plotters
        if(exp->load())
        {           
            //adjust plotpicker and timescale
            for(int i = 0; i<5; i++)
                plotter[i]->updatePlotPicker(exp->data->utc);

            //enable options
            if(!ui->scrollArea->isEnabled())
            {
                ui->scrollArea->setEnabled(true);
                ui->coordPlot->setEnabled(true);
                ui->speedPlot->setEnabled(true);
                ui->accPlot->setEnabled(true);
                ui->angAccPlot->setEnabled(true);
                ui->OBDPlot->setEnabled(true);

                ui->coordPlot->installEventFilter(this);
                ui->speedPlot->installEventFilter(this);
                ui->accPlot->installEventFilter(this);
                ui->angAccPlot->installEventFilter(this);
                ui->OBDPlot->installEventFilter(this);
            }

            //xdata
            delete xdata;
            xdata = new double[exp->data->length];
            for(int i = 0; i<exp->data->length; i++)
               xdata[i] = (double)i;

            //assign data to all plotters
            plotter[0]->setData(xdata,exp->data->altitude,2,exp->data->length);
            plotter[1]->setData(xdata,exp->data->vehicle_speed,0,exp->data->length);
            plotter[1]->setData(xdata,exp->data->course,1,exp->data->length);
            plotter[2]->setData(xdata,exp->data->x_acceleration,0,exp->data->length);
            plotter[2]->setData(xdata,exp->data->y_acceleration,1,exp->data->length);
            plotter[2]->setData(xdata,exp->data->z_acceleration,2,exp->data->length);
            plotter[3]->setData(xdata,exp->data->x_acceleration_gyro,0,exp->data->length);
            plotter[3]->setData(xdata,exp->data->y_acceleration_gyro,1,exp->data->length);
            plotter[3]->setData(xdata,exp->data->z_acceleration_gyro,2,exp->data->length);
            plotter[4]->setData(xdata,exp->data->vehicle_speed_obd,0,exp->data->length);
            plotter[4]->setData(xdata,exp->data->throttle_position_obd,1,exp->data->length);
            plotter[4]->setData(xdata,exp->data->engine_rpm_obd,2,exp->data->length);
            plotter[4]->setData(xdata,exp->data->actual_engine_torque_obd,3,exp->data->length);
            plotter[4]->setData(xdata,exp->data->engine_reference_torque_obd,4,exp->data->length);
            plotter[4]->setData(xdata,exp->data->intake_air_temperature_obd,5,exp->data->length);
            plotter[4]->setData(xdata,exp->data->ambient_air_temperature_obd,6,exp->data->length);
            plotter[4]->setData(xdata,exp->data->barometric_pressure_obd,7,exp->data->length);

            //adjust spinboxes
            ui->spinCoordBegin->blockSignals(true);
            ui->spinCoordEnd->blockSignals(true);
            ui->spinCoordEnd->setMaximum(exp->data->length-1);
            ui->spinCoordBegin->setMaximum(exp->data->length);
            ui->spinCoordEnd->setValue(exp->data->length-1);
            ui->spinCoordBegin->setValue(0);
            ui->spinCoordBegin->blockSignals(false);
            ui->spinCoordEnd->blockSignals(false);

            //scale plotters
            for(int i = 0; i<5; i++)
                plotter[i]->rescale(ui->spinCoordBegin->value(), ui->spinCoordEnd->value());

            //reset zoom state
            for(int i = 0; i<5; i++)
                plotter[i]->resetZoom();

            //set vehicle id
            ui->vid->setText(exp->data->vid);

            //update map route
            gpsmap->setRoute(exp->data->longitude,exp->data->latitude,ui->spinCoordBegin->value(),ui->spinCoordEnd->value()+1);
            gpsmap->removeCar();
        }

        //done
        ui->statusBar->showMessage(tr("Ready"),0);
        loading = false;
    }
    return 0;
}

/**Updates thumbnail view when window size changed or the user double clicks a plot.*/
bool MainWindow::eventFilter(QObject *obj, QEvent *event)
{
    if(obj == this && event->type() == QEvent::Resize)
    {
        thumbviewer->updateView(ui->graphicsView->horizontalScrollBar()->sliderPosition());
        return false;
    }

    else if(obj == ui->coordPlot && event->type() == QEvent::MouseButtonDblClick)
    {
        if(int(plotter[0]->getXValue()) < 0){
            thumbviewer->jumpToPicture(exp->data->utc->at(0));
            gpsmap->setCar(exp->data->longitude[0],
                       exp->data->latitude[0]);
        }
        else if(int(plotter[0]->getXValue()) > exp->data->length){
            thumbviewer->jumpToPicture(exp->data->utc->at(exp->data->length-1));
            gpsmap->setCar(exp->data->longitude[exp->data->length-1],
                       exp->data->latitude[exp->data->length-1]);
        }
        else{
            thumbviewer->jumpToPicture(exp->data->utc->at(int(plotter[0]->getXValue())));
            gpsmap->setCar(exp->data->longitude[int(plotter[0]->getXValue())],
                       exp->data->latitude[int(plotter[0]->getXValue())]);
        }
    }

    else if(obj == ui->speedPlot && event->type() == QEvent::MouseButtonDblClick)
    {
        if(int(plotter[1]->getXValue()) < 0){
            thumbviewer->jumpToPicture(exp->data->utc->at(0));
            gpsmap->setCar(exp->data->longitude[0],
                       exp->data->latitude[0]);
        }
        else if(int(plotter[1]->getXValue()) > exp->data->length){
            thumbviewer->jumpToPicture(exp->data->utc->at(exp->data->length-1));
            gpsmap->setCar(exp->data->longitude[exp->data->length-1],
                       exp->data->latitude[exp->data->length-1]);
        }
        else{
            thumbviewer->jumpToPicture(exp->data->utc->at(int(plotter[1]->getXValue())));
            gpsmap->setCar(exp->data->longitude[int(plotter[1]->getXValue())],
                       exp->data->latitude[int(plotter[1]->getXValue())]);
        }
    }

    else if(obj == ui->accPlot && event->type() == QEvent::MouseButtonDblClick)
    {
        if(int(plotter[2]->getXValue()) < 0){
            thumbviewer->jumpToPicture(exp->data->utc->at(0));
            gpsmap->setCar(exp->data->longitude[0],
                       exp->data->latitude[0]);
        }
        else if(int(plotter[2]->getXValue()) > exp->data->length){
            thumbviewer->jumpToPicture(exp->data->utc->at(exp->data->length-1));
            gpsmap->setCar(exp->data->longitude[exp->data->length-1],
                       exp->data->latitude[exp->data->length-1]);
        }
        else{
            thumbviewer->jumpToPicture(exp->data->utc->at(int(plotter[2]->getXValue())));
            gpsmap->setCar(exp->data->longitude[int(plotter[2]->getXValue())],
                       exp->data->latitude[int(plotter[2]->getXValue())]);
        }
    }

    else if(obj == ui->angAccPlot && event->type() == QEvent::MouseButtonDblClick)
    {
        if(int(plotter[3]->getXValue()) < 0){
            thumbviewer->jumpToPicture(exp->data->utc->at(0));
            gpsmap->setCar(exp->data->longitude[0],
                       exp->data->latitude[0]);
        }
        else if(int(plotter[3]->getXValue()) > exp->data->length){
            thumbviewer->jumpToPicture(exp->data->utc->at(exp->data->length-1));
            gpsmap->setCar(exp->data->longitude[exp->data->length-1],
                       exp->data->latitude[exp->data->length-1]);
        }
        else{
            thumbviewer->jumpToPicture(exp->data->utc->at(int(plotter[3]->getXValue())));
            gpsmap->setCar(exp->data->longitude[int(plotter[3]->getXValue())],
                       exp->data->latitude[int(plotter[3]->getXValue())]);
        }
    }

    else if(obj == ui->OBDPlot && event->type() == QEvent::MouseButtonDblClick)
    {
        if(int(plotter[4]->getXValue()) < 0){
            thumbviewer->jumpToPicture(exp->data->utc->at(0));
            gpsmap->setCar(exp->data->longitude[0],
                       exp->data->latitude[0]);
        }
        else if(int(plotter[4]->getXValue()) > exp->data->length){
            thumbviewer->jumpToPicture(exp->data->utc->at(exp->data->length-1));
            gpsmap->setCar(exp->data->longitude[exp->data->length-1],
                       exp->data->latitude[exp->data->length-1]);
        }
        else{
            thumbviewer->jumpToPicture(exp->data->utc->at(int(plotter[4]->getXValue())));
            gpsmap->setCar(exp->data->longitude[int(plotter[4]->getXValue())],
                       exp->data->latitude[int(plotter[4]->getXValue())]);
        }
    }
    return false;
}

/**Displays the enlarged thumbnail.*/
void MainWindow::showPicture(QString fileName)
{
    //scale picture to fit to the window
    QPixmap pic(fileName);
    if(pic.width() > this->width()-100)
        pic = pic.scaledToWidth(this->width()-100);
    if(pic.height() > ui->tabWidget->height()-50)
        pic = pic.scaledToHeight(ui->tabWidget->height()-50);
    pictureLabel->setPixmap(pic);
    imageViewer->setWidget(pictureLabel);
    imageViewer->show();
    imageViewer->resize(pic.size());
    int x = this->x()+ui->widget->x()+(ui->widget->width()-imageViewer->width())/2;
    int y = this->y()+(this->frameSize().height()-imageViewer->height()-ui->graphicsView->height())/2;
    imageViewer->move(x,y);
}

/**Closes the enlarged thumbnail view.*/
void MainWindow::closePicture()
{
    imageViewer->close();
}

/**Places the car on the map when hovering over a thumbnail.*/
void MainWindow::highlighted(QString fileName)
{  
    QString name = fileName.right(19);
    name.remove(name.lastIndexOf("."),name.count()-name.lastIndexOf("."));
    QDateTime time = QDateTime::fromString(name,"yyyyMMdd_HHmmss");
    qint64 fileTime = time.toMSecsSinceEpoch();

    int a = 0;
    int b = exp->data->length;
    int index = 0;

    //binary search for closest data point
    while(b-a > 2)
    {
        qint64 time = QDateTime::fromString(exp->data->utc->at((b+a)/2),
                                            "yyyy-MM-dd HH:mm:ss:zzz").toMSecsSinceEpoch();
        if(fileTime > time)
            a = (b+a)/2;
        else
            b = (b+a)/2;

        if(b-a == 2)
            index = (b+a)/2;
        if(b-a == 1)
            index = a;
        if(b-a == 0)
            index = a;
    }

    //set the car
    gpsmap->setCar(exp->data->longitude[index],exp->data->latitude[index]);
}

