#ifndef THUMBNAILVIEWER_H
#define THUMBNAILVIEWER_H

#include <QtGui>

/** The thumbnailviewer class displays thumbnails. The user can view the pictures in a bigger size by clicking on them.*/
struct ScaledImageInfo{
    QImage scaledImage;
    QString scaledImagePath;
};

class thumbnailviewer: public QGraphicsView{
    Q_OBJECT

public:
    thumbnailviewer(QObject *parent,QGraphicsView *gui_view);

    ~thumbnailviewer();

    void loadImages(QDir *directory);

    void jumpToPicture(QString timeString);

    /**Contains the paths of all images in the currently loaded data set.*/
    QStringList fileList;

    /**Contians a pointer to the QGraphicsView the thumbnailviewer is installed on.*/
    QGraphicsView *view;

    /**The highlighting effect when the mouse hovers over a thumbnail.*/
    QGraphicsEffect *highlight;

    /**Contains a pointer to the MainWindow that created this thumbnailviewer.*/
    QObject *main;

public slots:

    void updateView(int pos);

private slots:

    void showImage(int num);

private:
    /**The eventFilter emits the closePicture signal when the mouse leaves the graphicsView and disables key inputs.*/
    bool eventFilter(QObject *, QEvent *);

    /**Becomes true when the user double clicked on the plot and jumps to thumbnails that might not have been loaded to the QGraphicsScene. */
    bool jumped;

    /**Contains the position of the ProxyWidget that has to be highlighted after the user double clicked a plot.*/
    int jumpedPosition;

    /**Contains the directory of the loaded data set's DCIM folder.*/
    QDir *dir;

    /**imageScaling handles loading and scaling thumbnails in a different thread.*/
    QFutureWatcher<ScaledImageInfo> *imageScaling;

    /**The thumbnails are added to this scene. The scene is set when the fileList is loaded and changes when the user changes the view.*/
    QGraphicsScene *scene;

    /**Contains all paths of images that are already loaded to the QGraphicsView*/
    QStringList loadedList;

    /**Indicates if the user enlarged a thumbnail. Becomes true when the user enlarges a thumbnail and false when the enlarged thumbnail gets closed.*/
    bool imageShown;

signals:
    /**closePicture is emitted when an enlarged thumbnail is shown and the mouse leaves the QGraphicsView*/
    void closePicture();
};



/** The ProxyWidget class provides containers for the thumbnails and handles mouse events */
class ProxyWidget: public QGraphicsProxyWidget{

    Q_OBJECT

public:
    ProxyWidget(thumbnailviewer *parent, QString targetFile, bool *shown);

    void paintEvent(QPaintEvent *);

protected:
    void mousePressEvent(QGraphicsSceneMouseEvent *event);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *);

    virtual void hoverLeaveEvent(QGraphicsSceneHoverEvent *);

private:
    /**Contains a pointer to the imageShown variable of the thumbnailviewer that created this ProxyWidget. Indicates if a thumbnail is enlarged.*/
    bool *imageShown;

    /**Contains a pointer to the thumbnailviewer that created this ProxyWidget.*/
    thumbnailviewer *viewer;

    /**Contains the path of the picture that is displayed in this ProxyWidget.*/
    QString file;

signals:
    /**pictureRequest is emitted when the user clicks on a thumbnail to enlarge it.*/
    void pictureRequest(QString fileName);

    /**closeRequest is emitted when the user clicks on a thumbnail to close the enlarged view.*/
    void closeRequest();

    /**highlihgtSignal is emitted when a thumbnail gets highlighted. The mainwindow sets the car symbol in the map to the point where the thumbnail was taken.*/
    void highlightSignal(QString fileName);
};



/** The HighlightEffect class provides the highlighting effect for enlarged thumbnails. */
class HighlightEffect : public QGraphicsEffect{
   Q_OBJECT

public:
    HighlightEffect();

    virtual QRectF  boundingRectFor(const QRectF &sourceRect) const;

protected:
    virtual void draw(QPainter *painter);
};

#endif // THUMBNAILVIEWER_H
