/*Copyright (C) 2012 Johannes Hiller, Michael Hoppe, Waldemar Heinrich
* Copyright (C) 2009 The Android Open Source Project
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*   you may not use this file except in compliance with the License.
*   You may obtain a copy of the License at
*
*       http://www.apache.org/licenses/LICENSE-2.0
*
*   Unless required by applicable law or agreed to in writing, software
*   distributed under the License is distributed on an "AS IS" BASIS,
*   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*   See the License for the specific language governing permissions and
*   limitations under the License.
*/

package de.tui.itlogger;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.UUID;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

/**
 * This class does all the work for setting up and managing Bluetooth
 * connections with other devices. It has a thread that listens for
 * incoming connections, a thread for connecting with a device, and a
 * thread for performing data transmissions when connected. It also analyzes the received data, converts it to integer
 * and post it to the Handler of OBD2_Activity.
 */
public class OBD2_Service {
    // Debugging
    private static final String TAG = "OBD2_Service";
    private static final boolean D = true;

    // UUID for SPP
    private static final UUID MY_UUID =
        UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");
   

    // Member fields
    private final BluetoothAdapter mAdapter;
    private final Handler mHandler;
    private ConnectThread mConnectThread;
    private ConnectedThread mConnectedThread;
    private int mState;

    // Constants that indicate the current connection state
    public static final int STATE_NONE = 0;       // we're doing nothing
    public static final int STATE_CONNECTING = 1; // now initiating an outgoing connection
    public static final int STATE_CONNECTED = 2;  // now connected to a remote device

    
    public static String VID;
    public static BluetoothDevice device_old;
    public static int errorcountconnect=0;
    /**
     * Constructor. Prepares a new OBD2_Activity session.
     * @param context  The UI Activity Context
     * @param handler  A Handler to send messages back to the UI Activity
     */
    public OBD2_Service(Context context, Handler handler) {
        mAdapter = BluetoothAdapter.getDefaultAdapter();
        mState = STATE_NONE;
        mHandler = handler;
    }

    /**
     * Set the current state of the chat connection
     * @param state  An integer defining the current connection state
     */
    private synchronized void setState(int state) {
        
        mState = state;

        // Give the new state to the Handler so the UI Activity can update
        mHandler.obtainMessage(OBD2_Activity.MESSAGE_STATE_CHANGE, state, -1).sendToTarget();
    }

    /**
     * Return the current connection state. */
    public synchronized int getState() {
        return mState;
    }

    /**
     * Start the chat service. Specifically start AcceptThread to begin a
     * session in listening (server) mode. Called by the Activity onResume() */
    public synchronized void start() {
        if (D) Log.d(TAG, "start");

        // Cancel any thread attempting to make a connection
        if (mConnectThread != null) {mConnectThread.cancel(); mConnectThread = null;}

        // Cancel any thread currently running a connection
        if (mConnectedThread != null) {mConnectedThread.cancel(); mConnectedThread = null;}
   
    }

    /**
     * Start the ConnectThread to initiate a connection to a remote device.
     * @param device  The BluetoothDevice to connect
     * @param secure Socket Security type - Secure (true)
     */
    public synchronized void connect(BluetoothDevice device, boolean secure) {
        if (D) Log.d(TAG, "connect to: " + device);
        device_old = device;

        // Cancel any thread attempting to make a connection
        if (mState == STATE_CONNECTING) {
            if (mConnectThread != null) {mConnectThread.cancel(); mConnectThread = null;}
        }

        // Cancel any thread currently running a connection
        if (mConnectedThread != null) {mConnectedThread.cancel(); mConnectedThread = null;}

        // Start the thread to connect with the given device
        mConnectThread = new ConnectThread(device, secure);
        mConnectThread.start();
        setState(STATE_CONNECTING);
    }

    /**
     * Start the ConnectedThread to begin managing a Bluetooth connection
     * @param socket  The BluetoothSocket on which the connection was made
     * @param device  The BluetoothDevice that has been connected
     */
    public synchronized void connected(BluetoothSocket socket, BluetoothDevice
            device, final String socketType) {
        if (D) Log.d(TAG, "connected, Socket Type:" + socketType);

        // Cancel the thread that completed the connection
        if (mConnectThread != null) {mConnectThread.cancel(); mConnectThread = null;}

        // Cancel any thread currently running a connection
        if (mConnectedThread != null) {mConnectedThread.cancel(); mConnectedThread = null;}

        // Cancel the accept thread because we only want to connect to one device
       

        // Start the thread to manage the connection and perform transmissions
        mConnectedThread = new ConnectedThread(socket, socketType);
        mConnectedThread.start();

        // Send the name of the connected device back to the UI Activity
        Message msg = mHandler.obtainMessage(OBD2_Activity.MESSAGE_DEVICE_NAME);
        Bundle bundle = new Bundle();
        bundle.putString(OBD2_Activity.DEVICE_NAME, device.getName());
        msg.setData(bundle);
        mHandler.sendMessage(msg);

        setState(STATE_CONNECTED);
        errorcountconnect=0;
    }

    /**
     * Stop all threads
     */
    public synchronized void stop() {
        if (D) Log.d(TAG, "stop");

        if (mConnectThread != null) {
            mConnectThread.cancel();
            mConnectThread = null;
        }

        if (mConnectedThread != null) {
            mConnectedThread.cancel();
            mConnectedThread = null;
        }
        
        setState(STATE_NONE);
    }

    /**
     * Write to the ConnectedThread in an unsynchronized manner
     * @param out The bytes to write
     * @see ConnectedThread#write(byte[])
     */
    public void write(byte[] out) {
        // Create temporary object
        ConnectedThread r;
        // Synchronize a copy of the ConnectedThread
        synchronized (this) {
            if (mState != STATE_CONNECTED) return;
            r = mConnectedThread;
        }
        // Perform the write unsynchronized
        r.write(out);
    }

    /**
     * Indicate that the connection attempt failed and notify the UI Activity.
     */
    private void connectionFailed() {
    	
    	if(OBD2_Activity.noreconnect == false){
    	try {
			Thread.sleep(2000);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    	errorcountconnect++;
    	
    	Log.i(TAG, "errorcountconnect:" + errorcountconnect);	
       
       if(errorcountconnect<4){
    	   
    	   
           connect(device_old, true);
           }else{
        	   
        	   mHandler.obtainMessage(OBD2_Activity.MESSAGE_UNABLE).sendToTarget();
            // Start the service over to restart listening mode
               OBD2_Service.this.start();
               errorcountconnect=0;
             
           	
           }
    	}
      
    }

    /**
     * Indicate that the connection was lost and notify the UI Activity.
     */
    private void connectionLost() {
    	
    	
    	mHandler.obtainMessage(OBD2_Activity.MESSAGE_NOCONNECTION).sendToTarget();
    	
    	if(OBD2_Activity.noreconnect == false){
    	//mHandler.obtainMessage(OBD2_Activity.MESSAGE_NOCONNECTION).sendToTarget();
    	try {
			Thread.sleep(2000);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    	errorcountconnect++;
    	Log.i(TAG, "errorcountconnect:" + errorcountconnect);	
       
        if(errorcountconnect<4){
        
		
        connect(device_old, true);
        }else{
        	
        	
         // Start the service over to restart listening mode
            OBD2_Service.this.start();
            errorcountconnect=0;
        	
        }
        
    	}
    }

    /**
     * This thread runs while attempting to make an outgoing connection
     * with a device. It runs straight through; the connection either
     * succeeds or fails.
     */
    private class ConnectThread extends Thread {
        private final BluetoothSocket mmSocket;
        private final BluetoothDevice mmDevice;
        private String mSocketType;

        public ConnectThread(BluetoothDevice device, boolean secure) {
            mmDevice = device;
            BluetoothSocket tmp = null;
            mSocketType = "Secure";

            // Get a BluetoothSocket for a connection with the
            // given BluetoothDevice
            try {
                if (secure) {
                    tmp = device.createRfcommSocketToServiceRecord(
                            MY_UUID);
                } 
                
            } catch (IOException e) {
                Log.e(TAG, "Socket Type: " + mSocketType + "create() failed", e);
            }
            mmSocket = tmp;
        }

        public void run() {
            Log.i(TAG, "BEGIN mConnectThread SocketType:" + mSocketType);
            setName("ConnectThread" + mSocketType);

            // Always cancel discovery because it will slow down a connection
            mAdapter.cancelDiscovery();

            // Make a connection to the BluetoothSocket
            try {
                // This is a blocking call and will only return on a
                // successful connection or an exception
                mmSocket.connect();
            } catch (IOException e) {
                // Close the socket
                try {
                    mmSocket.close();
                } catch (IOException e2) {
                    Log.e(TAG, "unable to close() " + mSocketType +
                            " socket during connection failure", e2);
                }
                connectionFailed();
                return;
            }

            // Reset the ConnectThread because we're done
            synchronized (OBD2_Service.this) {
                mConnectThread = null;
            }

            // Start the connected thread
            connected(mmSocket, mmDevice, mSocketType);
        }

        public void cancel() {
            try {
                mmSocket.close();
            } catch (IOException e) {
                Log.e(TAG, "close() of connect " + mSocketType + " socket failed", e);
            }
        }
    }

    /**
     * This thread runs during a connection with a remote device.
     * It handles all communication an analyzes incoming messages.  
     */
    private class ConnectedThread extends Thread {
        private final BluetoothSocket mmSocket;
        private final InputStream mmInStream;
        private final OutputStream mmOutStream;

        public ConnectedThread(BluetoothSocket socket, String socketType) {
            Log.d(TAG, "create ConnectedThread: " + socketType);
            mmSocket = socket;
            InputStream tmpIn = null;
            OutputStream tmpOut = null;

            // Get the BluetoothSocket input and output streams
            try {
                tmpIn = socket.getInputStream();
                tmpOut = socket.getOutputStream();
            } catch (IOException e) {
                Log.e(TAG, "temp sockets not created", e);
            }

            mmInStream = tmpIn;
            mmOutStream = tmpOut;
        }

        public void run() {
            Log.i(TAG, "BEGIN mConnectedThread");

            // Keep listening to the InputStream while connected
            while (true) {
                try {
                	
                	byte[] raw_buffer = new byte[1024];	
                	 int[] head = new int[2];
                     int bytes=0,buffpos=0, data=0;
                     
                    
                     // listen to the stream until ">" ignore " " and CR
                	
                	buffpos=0;
                	while (((char) (bytes = (byte) mmInStream.read()) != '>') ){
                		
                			if (((char) bytes != ' ') && ((char) bytes != 13)){
                				
            				raw_buffer[buffpos] =  (byte) bytes;
            				buffpos++;
            				
            				
                			}
	
                	}
                
                	// lookup for ERROR
               String readMessage = new String(raw_buffer, 0, buffpos);
               if((readMessage.contains("NO")==false) && (readMessage.contains("UNABLE")==false)){
            	   
            	   // lookup head
               if((raw_buffer[0] > '/') && (raw_buffer[0] < ':') && (buffpos > 3)){
                   	
            	   
                   String help1=null, help2=null, help3=null, help4=null;	
                   
                   help1=Character.toString ((char) raw_buffer[0]);
                   help2=Character.toString ((char) raw_buffer[1]);
                   help3=Character.toString ((char) raw_buffer[2]);
                   help4=Character.toString ((char) raw_buffer[3]);
                   
                   head[0]=Integer.parseInt(help1+help2,16);
                   head[1]=Integer.parseInt(help3+help4,16);
                	
                	
                   	
                	Log.i(TAG, "stelle 0:" + head[0]);	
               	 	Log.i(TAG, "stelle 1:" + head[1]);	
               	 	
               	 	// message starts with 09
               	 	if(head[0] == 0x09){
               	 		
               	 	VID=getVID(raw_buffer, buffpos);
               	 		
               	 	}
                   	
                
                    // message starts with 01 -> PID-request answer 
                    if (head [0] == 0x01){

                    	//RPM
                    	if(head [1] == OBD2_Activity.Value_EngineRPM){
                    		
                    	//get DATA from ASCII coded HEX
                    	data=getDATA_2Byte(raw_buffer, buffpos);
                    	 //post to handler of OBD2_Activity
                    	if(data != -255)
                    	mHandler.obtainMessage(OBD2_Activity.Value_EngineRPM, data).sendToTarget();
                    	}
                    	
                    	//ThrottlePosition
                    	else if(head[1] == OBD2_Activity.Value_ThrottlePosition){
                    		
                    		data=getDATA_1Byte(raw_buffer, buffpos);
                    		if(data != -255)
                            mHandler.obtainMessage(OBD2_Activity.Value_ThrottlePosition, data).sendToTarget();
                    			
                    	}
                    	
                    	//Speed
                    	else if(head[1] == OBD2_Activity.Value_VehicleSpeed){
                    		
                    		data=getDATA_1Byte(raw_buffer, buffpos);
                    		if(data != -255)
                            mHandler.obtainMessage(OBD2_Activity.Value_VehicleSpeed, data).sendToTarget();
                    			
                    	}
                    	
                    	//EnginepercentTorque
                    	else if(head[1] == OBD2_Activity.Value_ActualEnginePercentTorque){
                    		
                    		data=getDATA_1Byte(raw_buffer, buffpos);
                    		if(data != -255)
                            mHandler.obtainMessage(OBD2_Activity.Value_ActualEnginePercentTorque, data).sendToTarget();
                    			
                    	}
                    	
                    	//AirTemperature
                    	else if(head[1] == OBD2_Activity.Value_AmbientAirTemperature){
                    		

                    		data=getDATA_1Byte(raw_buffer, buffpos);
                    		if(data != -255)
                            mHandler.obtainMessage(OBD2_Activity.Value_AmbientAirTemperature, data).sendToTarget();
                    			
                    	}
                    	
                    	//BarometricPressure
                    	else if(head[1] == OBD2_Activity.Value_BarometricPressure){
                    		
                    		data=getDATA_1Byte(raw_buffer, buffpos);
                    		if(data != -255)
                            mHandler.obtainMessage(OBD2_Activity.Value_BarometricPressure, data).sendToTarget();
                    			
                    	}
                    	
                    	//ReferenceTorque
                    	else if(head[1] == OBD2_Activity.Value_EngineReferenceTorque){
                    		
                    		data=getDATA_2Byte(raw_buffer, buffpos);
                    		if(data != -255)
                            mHandler.obtainMessage(OBD2_Activity.Value_EngineReferenceTorque, data).sendToTarget();
                    			
                    	}
                    	
                    	//IntakeAirTemperature
                    	else if(head[1] == OBD2_Activity.Value_IntakeAirTemperature){
                    		
                    		data=getDATA_1Byte(raw_buffer, buffpos);
                    		if(data != -255)
                            mHandler.obtainMessage(OBD2_Activity.Value_IntakeAirTemperature, data).sendToTarget();
                    			
                    		}
                    	//Support mask PID 1-20
                    	else if(head[1] == OBD2_Activity.Support_1_20){
                    		

                    		boolean[] respond=getDATA_BIT(raw_buffer, buffpos);
                    		if(respond != null)
                            mHandler.obtainMessage(OBD2_Activity.Support_1_20, respond).sendToTarget();
                    			
                    		}
                    	//Support maske PID 21-40
                    	else if(head[1] == OBD2_Activity.Support_21_40){
                    		
                    		boolean[] respond=getDATA_BIT(raw_buffer, buffpos);
                    		if(respond != null)
                            mHandler.obtainMessage(OBD2_Activity.Support_21_40, respond).sendToTarget();
                    			
                    		}
                    	//Support maske PID 41-60
                    	else if(head[1] == OBD2_Activity.Support_41_60){
                    		
                    		boolean[] respond=getDATA_BIT(raw_buffer, buffpos);
                    		if(respond != null)
                            mHandler.obtainMessage(OBD2_Activity.Support_41_60, respond).sendToTarget();
                    			
                    		}
                    	//Support maske PID 61-80
                    	else if(head[1] == OBD2_Activity.Support_61_80){
                    		
                    		boolean[] respond=getDATA_BIT(raw_buffer, buffpos);
                    		if(respond != null)
                            mHandler.obtainMessage(OBD2_Activity.Support_61_80, respond).sendToTarget();
                    			
                    		}
                    }
                    
                }else{
                	
                	// answer to reset Message (only message starts with no HEX)
              	   mHandler.obtainMessage(OBD2_Activity.Startmessage, readMessage).sendToTarget();
              	   	
                	} 	
          		}else{
                	// on error
          			mHandler.obtainMessage(OBD2_Activity.MESSAGE_INITIALOBDERROR).sendToTarget();

                }
                	
                	
                } catch (IOException e) {
                    Log.e(TAG, "disconnected", e);
                    mHandler.obtainMessage(OBD2_Activity.MESSAGE_NOCONNECTION).sendToTarget();
                    connectionLost();
                    break;
                }
            }
        }

        
      public String getVID(byte[] buffer, int bufferpos){
    	  
    	  String VID = "", help1=null, help2=null;
    	  int count=0;
    	  
     	char help_char;
		int offset=8;
     	 
 		 // check if there are no VIN digits within the last 8 digits
 		if(bufferpos > 7){ 
		for(int i=bufferpos-8; i<bufferpos; i++){
 			 
 	    	  count++;
    		  if(help1==null){
    	  help1=Character.toString ((char) buffer[i]);
    		  }else{
    	  help2=Character.toString ((char) buffer[i]); 
    		  }
    		  
    		  if(count == 2){
    			
    			help_char=((char) Integer.parseInt(help1+help2,16));
    			
    			// check if it only contains numbers or capital letters
    			if ( ((help_char > '/') && (help_char < ':')) || ((help_char > '@') && (help_char < '[')) ){
    				
    				offset=offset-2;
    			}
    			
    			count=0;
    			help1=null;
    			help2=null;
    			    
    		  }
 			 
 			 
 		 }
 		}else{
 			geterror();
 		}

     	// VIN of CAN-BUS
 		try{
     	 if(buffer[bufferpos-(45+offset)] == ':'){
     	 
     	  for(int i=bufferpos-(38+offset); i < (bufferpos-offset); i++){
     		  
     		  if(i < bufferpos-(1+offset)){
     		  	if(buffer[i+1] == ':'){
     		  		i=i+2;
     		  	}
     		  }
     		  
     		 
      			  count++;
      		  if(help1==null){
      			  help1=Character.toString ((char) buffer[i]);
      			Log.i(TAG, "help1 :" + help1);
      		  }else{
      			  help2=Character.toString ((char) buffer[i]); 
      			Log.i(TAG, "help2 :" + help2);
      		  }
      		
      		  if(count == 2){
      			
      			VID=VID+((char) Integer.parseInt(help1+help2,16));
      			
      			count=0;
      			help1=null;
      			help2=null;
      			    
      		  }
      	 
      	  }
     	
     		  
     	  
     	 }else{//VIN as 5 line matrix
     		 
     		 // matrix line by line request
    	  VID=getVID_PART(bufferpos-(64+offset), bufferpos-(56+offset), buffer, bufferpos);
    	  VID=VID+getVID_PART(bufferpos-(50+offset), bufferpos-(42+offset), buffer, bufferpos);
    	  VID=VID+getVID_PART(bufferpos-(36+offset), bufferpos-(28+offset), buffer, bufferpos);
    	  VID=VID+getVID_PART(bufferpos-(22+offset), bufferpos-(14+offset), buffer, bufferpos);
    	  VID=VID+getVID_PART(bufferpos-(8+offset), bufferpos-offset, buffer, bufferpos);
     	 }
     	 Log.i(TAG, "VID :" + VID);
    	  
    	  return VID;
 		}finally{
 			geterror();
 		}
      }
       
      // get the requested line
      public String getVID_PART(int start, int end, byte[] buffer, int buffpos){
    	  
    	  try{
    		  
    	  String VID_PART="", help1=null, help2=null;
    	  
    	  int count=0;
    	  
    	  for(int i=start; i < end; i++){
    		 
    		  
    	  count++;
    		  if(help1==null){
    	  help1=Character.toString ((char) buffer[i]);
    	  Log.i(TAG, "help1 :" + help1);
    		  }else{
    	  help2=Character.toString ((char) buffer[i]);
    	  Log.i(TAG, "help2 :" + help2);
    		  }
    		  
    		  if(count == 2){
    			if(Integer.parseInt(help1+help2,16) != 0){
    			VID_PART=VID_PART+((char) Integer.parseInt(help1+help2,16));
    			Log.i(TAG, "VID_PART :" + VID_PART);
    			}
    			count=0;
    			help1=null;
    			help2=null;
    			    
    		  }
    	 
    	  }
    	  
    	
    	  return VID_PART;
    	  }finally{
    		  geterror();
    	  }
      }
      
      /**
       * This method converts incoming 2 Byte HEX values into integer. 
       */
      
       public int getDATA_2Byte(byte[] buffer, int buffpos){

        	int data=0;
        	boolean ok=true;

        	String help1=null, help2=null, help3=null, help4=null;	
            
        	// stream must be longer then 3
        	if(buffpos > 3){
        	for(int i=1; i<5; i++){
        		
        		//should only contain numbers an capital latter's from A to F  
        		if(((buffer[buffpos-i] < '0') || (buffer[buffpos-i] > '9')) && ((buffer[buffpos-i] < 'A') || (buffer[buffpos-i] > 'F'))){
        			ok=false;
        			geterror();
        		}
        		
        	}
        	}else{
        		ok=false;
        		geterror();
        	}
        	
        	if(ok == true){
        	
        		// convert to string if data is ok
        	help1=Character.toString ((char) buffer[buffpos-4]);
            help2=Character.toString ((char) buffer[buffpos-3]);
            help3=Character.toString ((char) buffer[buffpos-2]);
            help4=Character.toString ((char) buffer[buffpos-1]);	
        		
       	 	// phrase integer value of this 
            data=Integer.parseInt(help1+help2+help3+help4,16);
            return data;	
        	}else{
        		//error
        		return -255;
        	}	
        }
       
       /**
        * This method converts incoming 1 Byte HEX values into integer. 
        */
       
       public int getDATA_1Byte(byte[] buffer, int buffpos){
      
       	int data=0;
       	boolean ok=true;
       	
       
    	String help1=null, help2=null;	
    	if(buffpos > 1){
    		for(int i=1; i<3; i++){
        		
        		if(((buffer[buffpos-i] < '0') || (buffer[buffpos-i] > '9')) && ((buffer[buffpos-i] < 'A') || (buffer[buffpos-i] > 'F'))){
        			ok=false;
        			geterror();
        		}
        		
        	}
        	}else{
        		ok=false;
        		geterror();
        	}
    	
    	if(ok == true){	
        help1=Character.toString ((char) buffer[buffpos-2]);
        help2=Character.toString ((char) buffer[buffpos-1]);	
        
   	 	
        data=Integer.parseInt(help1+help2,16);
        return data;	
    	}else{
    		return -255;
    	}
       	
        
	
       }
       
       /**
        * This method converts incoming message into bits. 
        */
       public boolean[] getDATA_BIT(byte[] buffer, int buffpos){
    	   
    	   
          	String help=null;
          	int k=0;
          
           boolean[] binary = new boolean[32];
          	
        // stream must be longer then 7
           if(buffpos > 7){
          	for (int i=buffpos-8;i < buffpos; i++){
          		
          	//should only contain numbers an capital latter's from A to F  
          		if ( ((buffer[i] > '/') && (buffer[i] < ':')) || ((buffer[i] > '@') && (buffer[i] < 'G')) )
          		{
          			//covert to string of 0 and 1
          		help=Integer.toBinaryString(Integer.decode("0x"+((char) buffer[i])));

          		// add 0's (leading zeros are not converted)
          		while(help.length()<4){
          			help='0'+help;
          			}
          		// get a bit-array of the string
          	    for (int j=0; j<help.length(); j++){
          	        if (help.charAt(j) == '1'){             
          	            binary[k] = true; 
          	            k++;
          	        	}
          	        else if (help.charAt(j) == '0'){
          	            binary[k] = false; 
          	            k++;
          	        	}
          	    
          	    	}
          		
          		}else{
          			geterror();
          			binary=null;
          			break;
          			}
          	}
           }else{
           	geterror();
      			binary=null;
           }
           
          	if(binary != null){
          		return binary;
          	}else{
          		return null;
          	}
       }
       
       public void geterror(){
    	   
    	  // if there is an error
       	   mHandler.obtainMessage(OBD2_Activity.MESSAGE_OBDERROR).sendToTarget();
    	   
       }
        
        /**
         * Write to the connected OutStream.
         * @param buffer  The bytes to write
         */
        public void write(byte[] buffer) {
            try {
                mmOutStream.write(buffer);

                // Share the sent message back to the UI Activity
                mHandler.obtainMessage(OBD2_Activity.MESSAGE_WRITE, -1, -1, buffer)
                        .sendToTarget();
            } catch (IOException e) {
                Log.e(TAG, "Exception during write", e);
            }
        }

        public void cancel() {
            try {
                mmSocket.close();
            } catch (IOException e) {
                Log.e(TAG, "close() of connect socket failed", e);
            }
        }
    }
}
